/*-----------------------------------------------------*/
/*     Name:               calend.h                    */
/*-----------------------------------------------------*/
/*  Aufgabe: Makros, Prototypen, Strukturen usw.       */
/*-----------------------------------------------------*/
/*   Copyright (C) 1993 - 2003 by Michael Schroeder    */
/*-----------------------------------------------------*/


/*
 * LIBASTRO is distributed under the GNU GENERAL PUBLIC LICENSE (GPL)
 * Version 2 (June 1991). See the "COPYING" file distributed with this
 * software for more info.
 */

#ifndef CALEND_H
# define CALEND_H

#if __cplusplus
extern "C" {
#endif


typedef enum
{
  ROMAN_NONES = 0,
  ROMAN_IDES,
  ROMAN_KALENDS,
} ROMANFIXPOINT;

#ifndef ASTRO_H
typedef struct		/* Datum */
{
  long   year;
  int    month;
  double day;		/* mit Tagesbruchzahl = Stunde/24 */
} DATE,	*pDATE;
#endif

#pragma pack(1)
typedef struct		/* modifizierte DATE-Struktur */
{
  long   year;
  short  month;
  short  era;
  double day;
} GENGOUDATE, *pGENGOUDATE;

typedef struct		/* modifizierte DATE-Struktur */
{
  long   year;		/* lineares Jahr 		*/
  short  month;		/* month = DATE.month & 0xffff	*/
  short  leap;		/* leap  = DATE.month >> 16	*/
  double day;		/* mit Tagesbruchzahl = Stunde/24 */
} LEAPDATE, *pLEAPDATE;

typedef struct		/* modifizierte DATE-Struktur */
{
  char alautun;		/* ueberlagert ab hier DATE.year */
  char kinchiltun;
  char calabtun;
  char pictun;
  char baktun;		/* ueberlagert ab hier DATE.month */
  char katun;
  char tun;
  char uinal;
  float kin;		/* ueberlagert ab hier DATE.day */
  char tzolkinsign;
  char tzolkinnum;
  char haabsign;
  char haabnum;
} MAYADATE, *pMAYADATE;

typedef struct		/* modifizierte DATE-Struktur */
{
  long   year;
  short  month;
  short  fix;		/* ROMANFIXPOINT */
  double day;
} ROMANDATE, *pROMANDATE;
#pragma pack()

#define CHINESE_CYCLE(y,c,n)	({c = idiv2(y, 60, &n) + 1; n++;})
#define CHINESE_YEAR(c,n)	(((c) - 1) * 60 + (n) - 1)


#define H_FIXDATE	0x0000		/* Vom Datum abhaengiger Festtag */
#define H_WEEKDAY	0x0001		/* Vom Wochentag abhaengiger Festtag */
#define H_ECCLES	0x0002		/* Von Ostern abhaengiger Festtag */
#define H_MOVABLE	0x0003		/* Bewegliche Festtage */
#define H_HOLIDAY	0x0004		/* Feiertag */
#define H_EASTER	0x0008		/* Ostertermin */


typedef struct		/* Festtag */
{
  long jd;		/* JD des Festtags */
  int mark;		/* Art des Festtags */
  const char *name;	/* Bezeichnung des Festtags */
} HOLIDAY, *pHOLIDAY;


#ifndef EXTERN
# if defined(__WIN32__) && !defined(__GNUC__)
#  define EXTERN 	extern __import
# else
#  define EXTERN	extern 
# endif
#endif

/*
 * RSX: In WIN-DLLs werden Daten als Zeiger auf den Inhalt uebergeben :-( 
 *
 * Durch die folgende Definition koennen die Daten importiert werden:
 *
 *  #define IMPORT_DLL_DATA
 */
#if !defined(__calend__) && defined(IMPORT_DLL_DATA)
# define armenian_day		(*armenian_day)
# define armenian_month		(*armenian_month)
# define chinese_zodiac		(*chinese_zodiac)
# define chinese_zodiac_de	(*chinese_zodiac_de)
# define chinese_zodiac_en	(*chinese_zodiac_en)
# define coptic_day		(*coptic_day)
# define coptic_month		(*coptic_month)
# define egyptian_month		(*egyptian_month)
# define ethiopic_day		(*ethiopic_day)
# define ethiopic_month		(*ethiopic_month)
# define french_day		(*french_day)
# define french_month		(*french_month)
# define gengou_era		(*gengou_era)
# define hebrew_day		(*hebrew_day)
# define hebrew_day_de		(*hebrew_day_de)
# define hebrew_day_en		(*hebrew_day_en)
# define hebrew_month		(*hebrew_month)
# define hebrew_month_de	(*hebrew_month_de)
# define hebrew_month_en	(*hebrew_month_en)
# define hindu_day		(*hindu_day)
# define hindu_month		(*hindu_month)
# define hindu2_day		(*hindu2_day)
# define hindu2_month		(*hindu2_month)
# define indian_month		(*indian_month)
# define islamic_day		(*islamic_day)
# define islamic_day_de		(*islamic_day_de)
# define islamic_day_en		(*islamic_day_en)
# define islamic_month		(*islamic_month)
# define islamic_month_de	(*islamic_month_de)
# define islamic_month_en	(*islamic_month_en)
# define mayan_haabsign		(*mayan_haabsign)
# define mayan_tzolkinsign	(*mayan_tzolkinsign)
# define persian_month		(*persian_month)
# define roman_day		(*roman_day)
# define roman_month		(*roman_month)
# define syrian_month		(*syrian_month)
# define yazdegerd_month	(*yazdegerd_month)
#endif

EXTERN const char *const armenian_day[];  /* Armenische Tagesnamen	*/
EXTERN const char *const armenian_month[];/* Armenische Monatsnamen	*/
EXTERN const char *const *chinese_zodiac;   /* Chinesische Jahresnamen	*/
EXTERN const char *const chinese_zodiac_de[];/* Chinesische Jahresnamen	*/
EXTERN const char *const chinese_zodiac_en[];/* Chinesische Jahresnamen	*/
EXTERN const char *const coptic_day[];	/* Koptische Tagesnamen		*/
EXTERN const char *const coptic_month[];/* Koptische Monatsnamen	*/
EXTERN const char *const egyptian_month[];/* Aegypt. Monatsnamen	*/
EXTERN const char *const ethiopic_day[];  /* Aethiopische Tagesnamen	*/
EXTERN const char *const ethiopic_month[];/* Aethiopische Monatsnamen	*/
EXTERN const char *const french_day[];	/* Franz. Tagesnamen		*/
EXTERN const char *const french_month[];/* Franz. Monatsnamen		*/
EXTERN const char *const gengou_era[];	/* Japan. Gengou-Namen		*/
EXTERN const char *const *hebrew_day;	/* Juedische Tagesnamen		*/
EXTERN const char *const hebrew_day_de[];/* Juedische Tagesnamen	*/
EXTERN const char *const hebrew_day_en[];/* Juedische Tagesnamen	*/
EXTERN const char *const hebrew_month_de[];/* Juedische Monatsnamen	*/
EXTERN const char *const hebrew_month_en[];/* Juedische Monatsnamen	*/
EXTERN const char *const *hebrew_month;	/* Zeiger auf jued. Monatsnamen	*/
EXTERN const char *const hindu_day[];	/* Hindu Tagesnamen		*/
EXTERN const char *const hindu_month[]; /* Hindu Monatsnamen		*/
EXTERN const char *const hindu2_day[];	/* Hindu Tagesnamen (solar)	*/
EXTERN const char *const hindu2_month[]; /* Hindu Monatsnamen (solar)	*/
EXTERN const char *const indian_month[];/* Indische Monatsnamen		*/
EXTERN const char *const *islamic_day;	/* Islamische Tagesnamen	*/
EXTERN const char *const islamic_day_de[];/* Islamische Tagesnamen	*/
EXTERN const char *const islamic_day_en[];/* Islamische Tagesnamen	*/
EXTERN const char *const islamic_month_de[];/* Islamische Monatsnamen	*/
EXTERN const char *const islamic_month_en[];/* Islamische Monatsnamen	*/
EXTERN const char *const *islamic_month;/* Zeiger auf islam. Monatsnamen*/
EXTERN const char *const mayan_haabsign[];	/* Maya-Monatsnamen	*/
EXTERN const char *const mayan_tzolkinsign[];/* Maya-Tzolkinnamen	*/
EXTERN const char *const persian_month[];/* Persische Monatsnamen	*/
EXTERN const char *const roman_day[];	/* Roemische Tageszeichen	*/
EXTERN const char *const roman_month[];	/* Roemische Monatsnamen	*/
EXTERN const char *const syrian_month[];/* Syrische Monatsnamen		*/
EXTERN const char *const yazdegerd_month[];/* Jezdegerd-Monatsnamen	*/


pDATE armenian_caldate(double jd, pDATE date);
int armenian_isleap(long year);
double armenian_jd(pDATE date);
double armenian_jd0(long year);
int armenian_sprintf(char *buf, double jd, int mode);
const char *aureus(long year);
pDATE buddhist_caldate(double JD, pDATE date);
int buddhist_isleap(long year);
double buddhist_jd(pDATE date);
double buddhist_jd0(long year);
int buddhist_sprintf(char *buf, double jd, int mode);
pDATE byzantine_caldate(double JD, pDATE date);
int byzantine_isleap(long year);
double byzantine_jd(pDATE date);
double byzantine_jd0(long year);
int byzantine_sprintf(char *buf, double jd, int mode);
long *calphase(int year, long phase[52]);
pDATE chinese_caldate(double jd, pDATE date);
double chinese_chushu(long y);
double chinese_dongzhi(long y);
int chinese_isleap(long year);
double chinese_jd(pDATE date);
double chinese_jd0(long year);
int chinese_sprintf(char *buf, double jd, int mode);
double chinese_xiazhi(long y);
pDATE coptic_caldate(double jd, pDATE date);
int coptic_isleap(long year);
double coptic_jd(pDATE date);
double coptic_jd0(long year);
int coptic_sprintf(char *buf, double jd, int mode);
int datenum(pDATE date);
const char *dominical(long year);
pDATE easter(pDATE date);
pDATE egyptian_caldate(double jd, pDATE date);
int egyptian_isleap(long year);
double egyptian_jd(pDATE date);
double egyptian_jd0(long year);
int egyptian_sprintf(char *buf, double jd, int mode);
int epact(long year);
pDATE ethiopic_caldate(double jd, pDATE date);
int ethiopic_isleap(long year);
double ethiopic_jd(pDATE date);
double ethiopic_jd0(long year);
int ethiopic_sprintf(char *buf, double jd, int mode);
pDATE french_caldate(double jd, pDATE date);
int french_isleap(long year);
double french_jd(pDATE date);
double french_jd0(long year);
int french_sprintf(char *buf, double jd, int mode);
pDATE gengou_caldate(double JD, pDATE date);
int gengou_isleap(long year);
double gengou_jd(pDATE date);
double gengou_jd0(long year);
int gengou_sprintf(char *buf, double jd, int mode);
pDATE gregorian_caldate(double JD, pDATE date);
double gregorian_easter(long year);
int gregorian_isleap(long year);
double gregorian_jd(pDATE date);
double gregorian_jd0(long year);
int gregorian_sprintf(char *buf, double jd, int mode);
pDATE hebrew_caldate(double jd, pDATE date);
double hebrew_kislev(long year);
int hebrew_isleap(long year);
double hebrew_jd(pDATE date);
double hebrew_jd0(long year);
int hebrew_sprintf(char *buf, double jd, int mode);
double hebrew_tevet(long year);
pDATE hindu_caldate(double jd, pDATE date);
int hindu_isleap(long year);
double hindu_jd(pDATE date);
double hindu_jd0(long year);
int hindu_sprintf(char *buf, double jd, int mode);
pDATE hindu2_caldate(double jd, pDATE date);
int hindu2_isleap(long year);
double hindu2_jd(pDATE date);
double hindu2_jd0(long year);
int hindu2_sprintf(char *buf, double jd, int mode);
pHOLIDAY holiday(int year, int *n);
int indiction(long year);
pDATE indian_caldate(double jd, pDATE date);
int indian_isleap(long year);
double indian_jd(pDATE date);
double indian_jd0(long year);
int indian_sprintf(char *buf, double jd, int mode);
pDATE islamic_caldate(double jd, pDATE date);
int islamic_isleap(long year);
double islamic_jd(pDATE date);
double islamic_jd0(long year);
double islamic_ramadan(long year);
int islamic_sprintf(char *buf, double jd, int mode);
int isleap(long year);
pDATE iso_caldate(double jd, pDATE date);
int iso_isleap(long year);
double iso_jd(pDATE date);
double iso_jd0(long year);
int iso_sprintf(char *buf, double jd, int mode);
long _jdcalref(long code);
double jdeaster(long year);
double jd_event(double start_jd, pDATE (*other_caldate)(double, pDATE), double (*other_jd)(long));
pDATE julian_caldate(double JD, pDATE date);
double julian_easter(long year);
int julian_isleap(long year);
double julian_jd(pDATE date);
double julian_jd0(long year);
int julian_sprintf(char *buf, double jd, int mode);
pDATE mayan_caldate(double jd, pDATE date);
double mayan_haab(long cycle);
int mayan_isleap(long y);
double mayan_jd(pDATE date);
double mayan_jd0(long cycle);
int mayan_sprintf(char *buf, double jd, int mode);
double mayan_tzolkin(long cycle);
int *monthlen(int year);
void num2date(int num, pDATE date);
pDATE persian_caldate(double jd, pDATE date);
int persian_isleap(long year);
double persian_jd(pDATE date);
double persian_jd0(long year);
int persian_sprintf(char *buf, double jd, int mode);
const char *regent(long year);
pDATE roman_caldate(double jd, pDATE date);
double roman_jd(pDATE date);
int roman_sprintf(char *buf, double jd, int mode);
pDATE seleucid_caldate(double JD, pDATE date);
int seleucid_isleap(long year);
double seleucid_jd(pDATE date);
double seleucid_jd0(long year);
int seleucid_sprintf(char *buf, double jd, int mode);
int suncirc(long year);
pDATE syrian_caldate(double jd, pDATE date);
int syrian_isleap(long year);
double syrian_jd(pDATE date);
double syrian_jd0(long year);
int syrian_sprintf(char *buf, double jd, int mode);
pDATE taiwanese_caldate(double JD, pDATE date);
int taiwanese_isleap(long year);
double taiwanese_jd(pDATE date);
double taiwanese_jd0(long year);
int taiwanese_sprintf(char *buf, double jd, int mode);
pDATE yazdegerd_caldate(double jd, pDATE date);
int yazdegerd_isleap(long year);
double yazdegerd_jd(pDATE date);
double yazdegerd_jd0(long year);
int yazdegerd_sprintf(char *buf, double jd, int mode);


#define roman_isleap	julian_isleap
#define roman_jd0	julian_jd0
#define roman_easter	julian_easter


#if __cplusplus
}
#endif


#endif		/* CALEND_H */



