/*
 * CALENDAR is distributed under the GNU GENERAL PUBLIC LICENSE (GPL)
 * Version 2 (June 1991). See the "COPYING" file distributed with this
 * software for more info.
 *
 * Copyright (C) 2000 - 2003 by Michael Schroeder <mdhs@imsch.de>
 */

//#define DEBUG
#define INCL_NLANGUAGE
#define INCL_USERINIT

#include <locale.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <time.h>
#include <unistd.h>

#include "fmath.h"
#include "astro.h"
#include "calend.h"
#ifdef DEBUG
# include "fmathctl.h"
#endif

#include "starcalc.h"
#include "version.h"
#include "calendar.h"



unsigned long _max_loop = MAX_LOOP;		/* Suchtiefe in Schleifen */

void (*addin_init_hook)(int) = init_addin;	/* Addin initialisieren */


/*
 * Hier folgen kurze Erlaeuterungen zu den Funktionen und Argumenten
 * in verschiedenen Sprachen (auf richtigen Zeichensatz achten) 
 */
static char *getText(int nResource)
{
  switch(nResource) {

    EXPLAIN(ARMENIAN_NAME,
	    "ARMENIAN",
	    "ARMENISCH");
    EXPLAIN(ARMENIAN_DESC,
	    "Converts an Armenian calendar date to a date.",
	    "Wandelt ein armenisches Kalenderdatum in ein Datum um.");
    EXPLAIN(ASTRONOMY_NAME,
	    "ASTRONOMICAL",
	    "ASTRONOMISCH");
    EXPLAIN(ASTRONOMY_DESC,
	    "Converts the astronomical day count (JD) to a date.",
	    "Wandelt die astronomische Tagesz"ae"hlung (JD) in ein Datum um.");
    EXPLAIN(BUDDHIST_NAME,
	    "BUDDHIST",
	    "BUDDHISTISCH");
    EXPLAIN(BUDDHIST_DESC,
	    "Converts a Buddhist calendar date to a date.",
	    "Wandelt ein buddhistisches Kalenderdatum in ein Datum um.");
    EXPLAIN(BYZANTINE_NAME,
	    "BYZANTINE",
	    "BYZANTINISCH");
    EXPLAIN(BYZANTINE_DESC,
	    "Converts a Byzantine calendar date to a date.",
	    "Wandelt ein byzantinisches Kalenderdatum in ein Datum um.");
    EXPLAIN(CALENDAR_NAME,
	    "CALENDAR",
	    "KALENDER");
    EXPLAIN(CALENDAR_DESC,
	    "Converts a date to a corresponding calendar date of that calendar.",
	    "Wandelt ein Datum in ein Kalenderdatum des ausgew"ae"hlten Kalenders um.");
    EXPLAIN(CHINESE_NAME,
	    "CHINESE",
	    "CHINESISCH");
    EXPLAIN(CHINESE_DESC,
	    "Converts a Chinese calendar date to a date.",
	    "Wandelt ein chinesisches Kalenderdatum in ein Datum um.");
    EXPLAIN(CHRONOLOGY_NAME,
	    "CHRONOLOGY",
	    "CHRONOLOGIE");
    EXPLAIN(CHRONOLOGY_DESC,
	    "Calculates for a given year one of the classical calendrical cycles.",
	    "Berechnet f"ue"r das Jahr einen der klassischen Kalenderzyklen.");
    EXPLAIN(COPTIC_NAME,
	    "COPTIC",
	    "KOPTISCH");
    EXPLAIN(COPTIC_DESC,
	    "Converts a Coptic calendar date to a date.",
	    "Wandelt ein koptisches Kalenderdatum in ein Datum um.");
    EXPLAIN(EGYPTIAN_NAME,
	    "EGYPTIAN",
	    AE"GYPTISCH");
    EXPLAIN(EGYPTIAN_DESC,
	    "Converts an Egyptian calendar date to a date.",
	    "Wandelt ein "ae"gyptisches Kalenderdatum in ein Datum um.");
    EXPLAIN(ETHIOPIC_NAME,
	    "ETHIOPIC",
	    AE"THIOPISCH");
    EXPLAIN(ETHIOPIC_DESC,
	    "Converts an Ethiopic calendar date to a date.",
	    "Wandelt ein "ae"thiopisches Kalenderdatum in ein Datum um.");
    EXPLAIN(EVENT_NAME,
	    "EVENT",
	    "EREIGNIS");
    EXPLAIN(EVENT_DESC,
	    "Converts an event of type 'nth day of month in the year' to "
	    " the corresponding calendar date.",
	    "Wandelt ein Ereignis der Art 'nter Tag im Monat des Jahres' in "
	    "das entsprechende Kalenderdatum um.");
    EXPLAIN(FRENCH_NAME,
	    "FRENCH",
	    "FRANZ"OE"SISCH");
    EXPLAIN(FRENCH_DESC,
	    "Converts a French calendar date to a date.",
	    "Wandelt ein franz"oe"sisches Kalenderdatum in ein Datum um.");
    EXPLAIN(GENGOU_NAME,
	    "GENGOU",
	    "GENGOU");
    EXPLAIN(GENGOU_DESC,
	    "Converts a Gengou calendar date to a date.",
	    "Wandelt ein Gengou-Kalenderdatum in ein Datum um.");
    EXPLAIN(GNOMON_NAME,
	    "GNOMON",
	    "GNOMON");
    EXPLAIN(GNOMON_DESC,
	    "Calculates the time which is displayed by the shadow stick of a sundial"
	    " (gnomon). At 0 a.m. of the day this agrees with the equation of time.",
	    "Berechnet die Uhrzeit, die der Schattenstab einer Sonnenuhr (Gnomon) "
	    "anzeigt. Um 0 Uhr des Tages entspricht dies der Zeitgleichung.");
    EXPLAIN(GREGORIAN_NAME,
	    "GREGORIAN",
	    "GREGORIANISCH");
    EXPLAIN(GREGORIAN_DESC,
	    "Converts a Gregorian calendar date to a date.",
	    "Wandelt ein gregorianisches Kalenderdatum in ein Datum um.");
    EXPLAIN(HEBREW_NAME,
	    "HEBREW",
	    "J"UE"DISCH");
    EXPLAIN(HEBREW_DESC,
	    "Converts a Hebrew calendar date to a date.",
	    "Wandelt ein j"ue"disches Kalenderdatum in ein Datum um.");
    EXPLAIN(HINDU_NAME,
	    "HINDU",
	    "HINDU");
    EXPLAIN(HINDU_DESC,
	    "Converts a Hindu calendar date (lunisolar) to a date.",
	    "Wandelt ein Hindu-Kalenderdatum (lunisolar) in ein Datum um.");
    EXPLAIN(HINDU2_NAME,
	    "HINDU2",
	    "HINDU2");
    EXPLAIN(HINDU2_DESC,
	    "Converts a Hindu calendar date (solar) to a date.",
	    "Wandelt ein Hindu-Kalenderdatum (solar) in ein Datum um.");
    EXPLAIN(HOLIDAY_NAME,
	    "HOLIDAY",
	    "FESTTAG");
    EXPLAIN(HOLIDAY_DESC,
	    "Computes an event (holiday) in a calendar. The date is the first "
	    "event within the year (cycle) of that calendar.",
	    "Berechnet ein Ereignis (Festtag) aus einem Kalender. Es wird das "
	    "n"ae"chstfolgende Ereignis innerhalb eines vorgegebenen "
	    "Jahres (Zyklus) aus dem entsprechenden Kalender berechnet.");
    EXPLAIN(HOLIDAY2_NAME,
	    "HOLIDAY2",
	    "FESTTAG2");
    EXPLAIN(HOLIDAY2_DESC,
	    "Computes an event (holiday) in a calendar. The date is the first "
	    "event that falls on or next after a given start date.",
	    "Berechnet ein Ereignis (Festtag) aus einem Kalender. Es wird das n"ae"chste "
	    "Ereignis berechnet, das auf ein vorgegebenes Startdatum folgt.");
    EXPLAIN(INDIAN_NAME,
	    "INDIAN",
	    "INDISCH");
    EXPLAIN(INDIAN_DESC,
	    "Converts an Indian calendar date to a date.",
	    "Wandelt ein indisches Kalenderdatum in ein Datum um.");
    EXPLAIN(ISLAMIC_NAME,
	    "ISLAMIC",
	    "ISLAMISCH");
    EXPLAIN(ISLAMIC_DESC,
	    "Converts an Islamic calendar date to a date.",
	    "Wandelt ein islamisches Kalenderdatum in ein Datum um.");
    EXPLAIN(ISO8601_NAME,
	    "ISO8601",
	    "ISO8601");
    EXPLAIN(ISO8601_DESC,
	    "Converts an ISO-8601 calendar date to a date.",
	    "Wandelt ein ISO-8601 Kalenderdatum in ein Datum um.");
    EXPLAIN(JULIAN_NAME,
	    "JULIAN",
	    "JULIANISCH");
    EXPLAIN(JULIAN_DESC,
	    "Converts a Julian calendar date to a date.",
	    "Wandelt ein julianisches Kalenderdatum in ein Datum um.");
    EXPLAIN(LEAPYEAR_NAME,
	    "LEAPYEAR",
	    "SCHALTJAHR");
    EXPLAIN(LEAPYEAR_DESC,
	    "Determines weather a year is a leap year within a calendar.",
	    "Ermittelt, ob ein Jahr innerhalb eines Kalenders ein Schaltjahr ist.");
    EXPLAIN(LEAPYEAR2_NAME,
	    "LEAPYEAR2",
	    "SCHALTJAHR2");
    EXPLAIN(LEAPYEAR2_DESC,
	    "Determines weather a date is within a leap year of that calendar.",
	    "Ermittelt, ob ein Datum in einem Schaltjahr des Kalenders liegt.");
    EXPLAIN(MAYAN_NAME,
	    "MAYAN",
	    "MAYA");
    EXPLAIN(MAYAN_DESC,
	    "Converts a Mayan calendar date to a date.",
	    "Wandelt ein Maya-Kalenderdatum in ein Datum um.");
    EXPLAIN(PERSIAN_NAME,
	    "PERSIAN",
	    "PERSISCH");
    EXPLAIN(PERSIAN_DESC,
	    "Converts an Persian calendar date to a date.",
	    "Wandelt ein persisches Kalenderdatum in ein Datum um.");
    EXPLAIN(ROMAN_NAME,
	    "LATIN",
	    "LATEINISCH");
    EXPLAIN(ROMAN_DESC,
	    "Converts a Latin (Roman) calendar date to a date.",
	    "Wandelt ein lateinisches (r"oe"misches) Kalenderdatum in ein Datum um.");
    EXPLAIN(SELEUCID_NAME,
	    "SELEUCID",
	    "SELEUKIDEN");
    EXPLAIN(SELEUCID_DESC,
	    "Converts a Seleucid calendar date to a date.",
	    "Wandelt ein Seleukiden-Kalenderdatum in ein Datum um.");
    EXPLAIN(TAIWANESE_NAME,
	    "TAIWANESE",
	    "TAIWANISCH");
    EXPLAIN(TAIWANESE_DESC,
	    "Converts a Taiwanese calendar date to a date.",
	    "Wandelt ein taiwanesisches Kalenderdatum in ein Datum um.");
    EXPLAIN(WHEN_NAME,
	    "WHEN",
	    "WANN");
    EXPLAIN(WHEN_DESC,
	    "Scans for the date of an event that is a combination of definite "
	    "characteristics: \nfix day of week, fix day of month and/or fix "
	    "month \n(e.g. Full Moon on Friday the 13th ;-)",
	    "Sucht nach dem Datum eines Ereignisses, das eine Kombination "
	    "bestimmter Merkmale erf"ue"llt: \nfester Wochentag, feste Tageszahl "
	    "und/oder feste Monatszahl \n(z.B. Vollmond am Freitag, dem 13. ;-)");
    EXPLAIN(WHEN2_NAME,
	    "WHEN2",
	    "WANN2");
    EXPLAIN(WHEN2_DESC,
	    "Scans for the occurence of two events on the same day.\n "
	    "(e.g. Easter and Passover)",
	    "Sucht nach dem Auftreten zweier Ereignisse am selben Tag. "
	    "(z.B. Ostern und Passah)");
    EXPLAIN(SYRIAN_NAME,
	    "SYRIAN",
	    "SYRISCH");
    EXPLAIN(SYRIAN_DESC,
	    "Converts a Syrian calendar date to a date.",
	    "Wandelt ein syrisches Kalenderdatum in ein Datum um.");
    EXPLAIN(YAZDEGERD_NAME,
	    "YAZDEGERD",
	    "JEZDEGERD");
    EXPLAIN(YAZDEGERD_DESC,
	    "Converts a Yazdegerd calendar date to a date.",
	    "Wandelt ein Jezdegerd-Kalenderdatum in ein Datum um.");

    EXPLAIN(CALTYPE_NAME,
	    "calendar",
	    "Kalender");
    EXPLAIN(CALTYPE_DESC,
	    "Name of calendar: Astronomical, Gregorian, Hebrew, Indian, "
	    "Islamic, ISO8601, Julian, Latin, Mayan...",
	    "Name des Kalenders: astronomisch, gregorianisch, indisch, "
	    "islamisch, ISO8601, j"ue"disch, lateinisch, maya...");
    EXPLAIN(CALTYPE1_NAME,
	    "1st calendar",
	    "1. Kalender");
    EXPLAIN(CALTYPE2_NAME,
	    "2nd calendar",
	    "2. Kalender");
    EXPLAIN(CALCYCLE_NAME,
	    "calendrical cycle",
	    "Kalenderzyklus");
    EXPLAIN(CALCYCLE_DESC,
	    "Name of the calendrical cycle: Epact, Golden Number, Indiction, "
	    "Regent, Solar Cycle, Dominical Letter.",
	    "Name des Kalenderzyklus: Epakte, Goldene Zahl, Indiktion, "
	    "Jahresregent, Sonnenzirkel, Sonntagsbuchstabe.");
    EXPLAIN(CALMODE_NAME,
	    "format mode",
	    "Formatmodus");
    EXPLAIN(CALMODE_DESC,
	    "Selects the output format: 0=number format, 1=string format",
	    "W"ae"hlt das Ausgabeformat: 0=Zahlenformat, 1=Zeichenformat");
    EXPLAIN(CELESTIAL_NAME,
	    "astronomical event",
	    "Astronom. Ereignis");
    EXPLAIN(CELESTIAL_DESC,
	    "Name of the astronomical event: New Moon... , Spring, Summer..., "
	    "Aries, Gemini... , Mars Opposition, Venus Inferior Conjunction...",
	    "Name des astronomischen Ereignisses: Neumond... , Sommeranfang... ,"
	    "Widder, Zwillinge..., Mars Opposition, Venus untere Konjunktion...");
    EXPLAIN(CYCLE_NAME,
	    "cycle",
	    "Zyklus");
    EXPLAIN(CYCLE_DESC,
	    "Number of the cycle.",
	    "Nummer des Zyklus.");
    EXPLAIN(DATE_NAME,
	    "date",
	    "Datum");
    EXPLAIN(DATE_DESC,
	    "Date",
	    "Datum");
    EXPLAIN(DAY_NAME,
	    "day",
	    "Tag");
    EXPLAIN(DAY_DESC,
	    "Number of day of month",
	    "Tageszahl");
    EXPLAIN(DAYNAME_NAME,
	    "name of day",
	    "Tagesname");
    EXPLAIN(DAYNAME_DESC,
	    "Name of day of month or number.",
	    "Tagesname oder Zahl.");
    EXPLAIN(EPOCH_NAME,
	    "epoch",
	    "Epoche");
    EXPLAIN(EPOCH_DESC,
	    "Year or number of cycle.",
	    "Jahreszahl oder Zyklusnummer.");
    EXPLAIN(FIXPOINT_NAME,
	    "division point",
	    "Festpunkt");
    EXPLAIN(FIXPOINT_DESC,
	    "Name of the division point: \"Nonas\", \"Ides\", \"Kalendas\".",
	    "Names des Festpunkts: \"Nonas\", \"Ides\", \"Kalendas\".");
    EXPLAIN(GENGOU_ERA_NAME,
	    "gengou era",
	    "Gengou-"AE"ra");
    EXPLAIN(GENGOU_ERA_DESC,
	    "Name of the gengou era or number.",
	    "Name der Gengou-"AE"ra oder Nummer.");
    EXPLAIN(HOLIDAYS_NAME,
	    "event",
	    "Ereignis");
    EXPLAIN(HOLIDAYS_DESC,
	    "Name of the event (holiday), e.g. New Year, Easter, ....",
	    "Name des Ereignisses (Festtag), z.B. Neujahr, Ostern, ....");
    EXPLAIN(HOLIDAYS1_NAME,
	    "1st event",
	    "1. Ereignis");
    EXPLAIN(HOLIDAYS2_NAME,
	    "2st event",
	    "2. Ereignis");
    EXPLAIN(LEAPMONTH_NAME,
	    "leap month",
	    "Schaltmonat");
    EXPLAIN(LEAPMONTH_DESC,
	    "TRUE if the month is a leap month, otherwise FALSE",
	    "WAHR wenn der Monat ein Schaltmonat ist, sonst FALSCH");
    EXPLAIN(MONTH_NAME,
	    "month",
	    "Monat");
    EXPLAIN(MONTH_DESC,
	    "Number of month",
	    "Monatszahl");
    EXPLAIN(MONTHNAME_NAME,
	    "month",
	    "Monat");
    EXPLAIN(MONTHNAME_DESC,
	    "Name of month or number",
	    "Monatsname oder Zahl");
    EXPLAIN(ORDINAL_NAME,
	    "ordinal",
	    "Ordnungszahl");
    EXPLAIN(ORDINAL_DESC,
	    "Ordinal number of the name of day: 1=first, 2=second, ..., "
	    "0=last, -1=before last, ...",
	    "Ordnungszahl des Tagesnamens: 1=erster, 2=zweiter, ..., "
	    "0=letzer, -1=vorletzter, ...");
    EXPLAIN(START_NAME,
	    "date",
	    "Datum");
    EXPLAIN(START_DESC,
	    "Start date",
	    "Startdatum");
    EXPLAIN(WEEK_NAME,
	    "week",
	    "Woche");
    EXPLAIN(WEEK_DESC,
	    "Number of the week",
	    "Nummer der Woche");
    EXPLAIN(YEAR_NAME,
	    "year",
	    "Jahr");
    EXPLAIN(YEAR_DESC,
	    "Year",
	    "Jahreszahl");
    EXPLAIN(BAKTUN_NAME,
	    "baktun",
	    "Baktun");
    EXPLAIN(BAKTUN_DESC,
	    "1 baktun = 20 katun = 144000 days.",
	    "1 Baktun = 20 Katun = 144000 Tage.");
    EXPLAIN(KATUN_NAME,
	    "katun",
	    "Katun");
    EXPLAIN(KATUN_DESC,
	    "1 katun = 20 tun = 7200 days.",
	    "1 Katun = 20 Tun = 7200 Tage.");
    EXPLAIN(TUN_NAME,
	    "tun",
	    "Tun");
    EXPLAIN(TUN_DESC,
	    "1 tun = 18 uinal = 360 days.",
	    "1 Tun = 18 Uinal = 360 Tage.");
    EXPLAIN(UINAL_NAME,
	    "uinal",
	    "Uinal");
    EXPLAIN(UINAL_DESC,
	    "1 uinal = 20 kin = 20 days.",
	    "1 Uinal = 20 Kin = 20 Tage.");
    EXPLAIN(KIN_NAME,
	    "kin",
	    "Kin");
    EXPLAIN(KIN_DESC,
	    "1 kin = 1 day.",
	    "1 Kin = 1 Tag.");

    default:
      break;
  }

  return("");
}


SO_CALLBACK(GetFunctionCount, USHORT *nCount)
{
  char *env;

#ifdef DEBUG
  onerr = _guipopup;			/* Rechenfehler melden */
#endif

  tzset();				/* Zeitzonen-Info laden */
  gettz();				/* Bibliothek initialisieren */

  if ((env = getenv("SOADDIN_CALREFORM")))	/* anderer Termin der Kalenderreform ? */ 
    _jdcalref(atol(env));

  if ((env = getenv("SOADDIN_DELTA_T")))	/* Wert fuer TD - UT ? */ 
    deltat = atol(env) * censec;	  	/* Wert Jahrhunderten */

  if ((env = getenv("SOADDIN_LANGUAGE"))) {	/* Sprache ? */ 
    switch (GET2(env)) {
      case MKNUM('D','E'):
       _nLanguage = LANGUAGE_GERMAN;
       break;

      case MKNUM('E','N'):
       _nLanguage = LANGUAGE_ENGLISH;
       break;
    }
  }

  if ((env = getenv("SOADDIN_MAX_LOOP")))	/* Suchtiefe ? */ 
    _max_loop = atol(env);

  if ((env = getenv("SOADDIN_TIMEZONE")))	/* Zeitzone ? */ 
    tzone = atol(env) * censec;	  	/* Wert Jahrhunderten */

  setlocale(LC_NUMERIC, "");		/* Dezimalpunkt einstellen */

  *nCount = COUNT_OF_FUNCS;
}


/*
 * Diese Funktion gibt Auskunft ueber die Zahl der Argumente, Datentypen und
 * Namen der exportierten Funktionen
 *
 * Folgende Parmeter muessen versorgt werden
 *
 * USHORT *nNo:	        Nummer der Funktion zwischen 0 und nCount - 1  
 * char   *pFuncName:   Name der Funktion, die in der SharedLibrary (DLL)
 *			verwendet werden soll
 * USHORT *nParamCount: Anzahl aller Parameter (maximal 16) einschliesslich
 *			des Rueckgabewertes
 * ParamType *peType:   Zeiger auf ein Array mit den Datentypen aller Parameter
 *			Es muessen alle 16 moeglichen Parameter enthalten sein!
 * char *pInternalName: Name der Funktion, die der StarCalc-User sehen soll  
 */
SO_QUIRKSCALL(GetFunctionData, USHORT *nNo, char *pFuncName,
		USHORT *nParamCount, ParamType *peType, char *pInternalName)
{
  switch(*nNo) {

    case FUNC_ARMENIAN:
      SO_StringCopy(pFuncName, Q(SC_armenian));
      SO_StringCopy(pInternalName, getText(ARMENIAN_NAME));
      peType[0] = PTR_DOUBLE;
      peType[1] = PTR_DOUBLE;
      peType[2] = PTR_STRING;
      peType[3] = PTR_DOUBLE;
      *nParamCount = 4;
      break;

    case FUNC_ASTRONOMY:
      SO_StringCopy(pFuncName, Q(SC_astronomy));
      SO_StringCopy(pInternalName, getText(ASTRONOMY_NAME));
      peType[0] = PTR_DOUBLE;
      peType[1] = PTR_DOUBLE;
      *nParamCount = 2;
      break;

    case FUNC_BUDDHIST:
      SO_StringCopy(pFuncName, Q(SC_buddhist));
      SO_StringCopy(pInternalName, getText(BUDDHIST_NAME));
      peType[0] = PTR_DOUBLE;
      peType[1] = PTR_DOUBLE;
      peType[2] = PTR_STRING;
      peType[3] = PTR_DOUBLE;
      *nParamCount = 4;
      break;

    case FUNC_BYZANTINE:
      SO_StringCopy(pFuncName, Q(SC_byzantine));
      SO_StringCopy(pInternalName, getText(BYZANTINE_NAME));
      peType[0] = PTR_DOUBLE;
      peType[1] = PTR_DOUBLE;
      peType[2] = PTR_STRING;
      peType[3] = PTR_DOUBLE;
      *nParamCount = 4;
      break;

    case FUNC_CALENDAR:
      SO_StringCopy(pFuncName, Q(SC_calendar));
      SO_StringCopy(pInternalName, getText(CALENDAR_NAME));
      peType[0] = PTR_STRING;
      peType[1] = PTR_STRING;
      peType[2] = PTR_DOUBLE;
      peType[3] = PTR_DOUBLE;
      *nParamCount = 4;
      break;

    case FUNC_CHINESE:
      SO_StringCopy(pFuncName, Q(SC_chinese));
      SO_StringCopy(pInternalName, getText(CHINESE_NAME));
      peType[0] = PTR_DOUBLE;
      peType[1] = PTR_DOUBLE;
      peType[2] = PTR_DOUBLE;
      peType[3] = PTR_STRING;
      peType[4] = PTR_DOUBLE;
      peType[5] = PTR_DOUBLE;
      *nParamCount = 6;
      break;

    case FUNC_CHRONOLOGY:
      SO_StringCopy(pFuncName, Q(SC_chronology));
      SO_StringCopy(pInternalName, getText(CHRONOLOGY_NAME));
      peType[0] = PTR_STRING;
      peType[1] = PTR_STRING;
      peType[2] = PTR_DOUBLE;
      *nParamCount = 3;
      break;

    case FUNC_COPTIC:
      SO_StringCopy(pFuncName, Q(SC_coptic));
      SO_StringCopy(pInternalName, getText(COPTIC_NAME));
      peType[0] = PTR_DOUBLE;
      peType[1] = PTR_DOUBLE;
      peType[2] = PTR_STRING;
      peType[3] = PTR_DOUBLE;
      *nParamCount = 4;
      break;

    case FUNC_EGYPTIAN:
      SO_StringCopy(pFuncName, Q(SC_egyptian));
      SO_StringCopy(pInternalName, getText(EGYPTIAN_NAME));
      peType[0] = PTR_DOUBLE;
      peType[1] = PTR_DOUBLE;
      peType[2] = PTR_STRING;
      peType[3] = PTR_DOUBLE;
      *nParamCount = 4;
      break;

    case FUNC_ETHIOPIC:
      SO_StringCopy(pFuncName, Q(SC_ethiopic));
      SO_StringCopy(pInternalName, getText(ETHIOPIC_NAME));
      peType[0] = PTR_DOUBLE;
      peType[1] = PTR_DOUBLE;
      peType[2] = PTR_STRING;
      peType[3] = PTR_DOUBLE;
      *nParamCount = 4;
      break;

    case FUNC_EVENT:
      SO_StringCopy(pFuncName, Q(SC_event));
      SO_StringCopy(pInternalName, getText(EVENT_NAME));
      peType[0] = PTR_DOUBLE;
      peType[1] = PTR_DOUBLE;
      peType[2] = PTR_STRING;
      peType[3] = PTR_STRING;
      peType[4] = PTR_DOUBLE;
      *nParamCount = 5;
      break;

    case FUNC_FRENCH:
      SO_StringCopy(pFuncName, Q(SC_french));
      SO_StringCopy(pInternalName, getText(FRENCH_NAME));
      peType[0] = PTR_DOUBLE;
      peType[1] = PTR_DOUBLE;
      peType[2] = PTR_STRING;
      peType[3] = PTR_DOUBLE;
      *nParamCount = 4;
      break;

    case FUNC_GENGOU:
      SO_StringCopy(pFuncName, Q(SC_gengou));
      SO_StringCopy(pInternalName, getText(GENGOU_NAME));
      peType[0] = PTR_DOUBLE;
      peType[1] = PTR_STRING;
      peType[2] = PTR_DOUBLE;
      peType[3] = PTR_STRING;
      peType[4] = PTR_DOUBLE;
      *nParamCount = 5;
      break;

    case FUNC_GNOMON:
      SO_StringCopy(pFuncName, Q(SC_gnomon));
      SO_StringCopy(pInternalName, getText(GNOMON_NAME));
      peType[0] = PTR_DOUBLE;
      peType[1] = PTR_DOUBLE;
      *nParamCount = 2;
      break;

    case FUNC_GREGORIAN:
      SO_StringCopy(pFuncName, Q(SC_gregorian));
      SO_StringCopy(pInternalName, getText(GREGORIAN_NAME));
      peType[0] = PTR_DOUBLE;
      peType[1] = PTR_DOUBLE;
      peType[2] = PTR_STRING;
      peType[3] = PTR_DOUBLE;
      *nParamCount = 4;
      break;

    case FUNC_HEBREW:
      SO_StringCopy(pFuncName, Q(SC_hebrew));
      SO_StringCopy(pInternalName, getText(HEBREW_NAME));
      peType[0] = PTR_DOUBLE;
      peType[1] = PTR_DOUBLE;
      peType[2] = PTR_STRING;
      peType[3] = PTR_DOUBLE;
      *nParamCount = 4;
      break;

    case FUNC_HINDU:
      SO_StringCopy(pFuncName, Q(SC_hindu));
      SO_StringCopy(pInternalName, getText(HINDU_NAME));
      peType[0] = PTR_DOUBLE;
      peType[1] = PTR_DOUBLE;
      peType[2] = PTR_STRING;
      peType[3] = PTR_DOUBLE;
      peType[4] = PTR_DOUBLE;
      *nParamCount = 5;
      break;

    case FUNC_HINDU2:
      SO_StringCopy(pFuncName, Q(SC_hindu2));
      SO_StringCopy(pInternalName, getText(HINDU2_NAME));
      peType[0] = PTR_DOUBLE;
      peType[1] = PTR_DOUBLE;
      peType[2] = PTR_STRING;
      peType[3] = PTR_DOUBLE;
      *nParamCount = 4;
      break;

    case FUNC_HOLIDAY:
      SO_StringCopy(pFuncName, Q(SC_holiday));
      SO_StringCopy(pInternalName, getText(HOLIDAY_NAME));
      peType[0] = PTR_DOUBLE;
      peType[1] = PTR_STRING;
      peType[2] = PTR_STRING;
      peType[3] = PTR_STRING;
      *nParamCount = 4;
      break;

    case FUNC_HOLIDAY2:
      SO_StringCopy(pFuncName, Q(SC_holiday2));
      SO_StringCopy(pInternalName, getText(HOLIDAY2_NAME));
      peType[0] = PTR_DOUBLE;
      peType[1] = PTR_STRING;
      peType[2] = PTR_STRING;
      peType[3] = PTR_DOUBLE;
      *nParamCount = 4;
      break;

    case FUNC_INDIAN:
      SO_StringCopy(pFuncName, Q(SC_indian));
      SO_StringCopy(pInternalName, getText(INDIAN_NAME));
      peType[0] = PTR_DOUBLE;
      peType[1] = PTR_DOUBLE;
      peType[2] = PTR_STRING;
      peType[3] = PTR_DOUBLE;
      *nParamCount = 4;
      break;

    case FUNC_ISLAMIC:
      SO_StringCopy(pFuncName, Q(SC_islamic));
      SO_StringCopy(pInternalName, getText(ISLAMIC_NAME));
      peType[0] = PTR_DOUBLE;
      peType[1] = PTR_DOUBLE;
      peType[2] = PTR_STRING;
      peType[3] = PTR_DOUBLE;
      *nParamCount = 4;
      break;

    case FUNC_ISO8601:
      SO_StringCopy(pFuncName, Q(SC_iso8601));
      SO_StringCopy(pInternalName, getText(ISO8601_NAME));
      peType[0] = PTR_DOUBLE;
      peType[1] = PTR_DOUBLE;
      peType[2] = PTR_DOUBLE;
      peType[3] = PTR_STRING;
      *nParamCount = 4;
      break;

    case FUNC_JULIAN:
      SO_StringCopy(pFuncName, Q(SC_julian));
      SO_StringCopy(pInternalName, getText(JULIAN_NAME));
      peType[0] = PTR_DOUBLE;
      peType[1] = PTR_DOUBLE;
      peType[2] = PTR_STRING;
      peType[3] = PTR_DOUBLE;
      *nParamCount = 4;
      break;

    case FUNC_LEAPYEAR:
      SO_StringCopy(pFuncName, Q(SC_leapyear));
      SO_StringCopy(pInternalName, getText(LEAPYEAR_NAME));
      peType[0] = PTR_DOUBLE;
      peType[1] = PTR_STRING;
      peType[2] = PTR_STRING;
      *nParamCount = 3;
      break;

    case FUNC_LEAPYEAR2:
      SO_StringCopy(pFuncName, Q(SC_leapyear2));
      SO_StringCopy(pInternalName, getText(LEAPYEAR2_NAME));
      peType[0] = PTR_DOUBLE;
      peType[1] = PTR_STRING;
      peType[2] = PTR_DOUBLE;
      *nParamCount = 3;
      break;

    case FUNC_MAYAN:
      SO_StringCopy(pFuncName, Q(SC_mayan));
      SO_StringCopy(pInternalName, getText(MAYAN_NAME));
      peType[0] = PTR_DOUBLE;
      peType[1] = PTR_DOUBLE;
      peType[2] = PTR_DOUBLE;
      peType[3] = PTR_DOUBLE;
      peType[4] = PTR_DOUBLE;
      peType[5] = PTR_DOUBLE;
      *nParamCount = 6;
      break;

    case FUNC_PERSIAN:
      SO_StringCopy(pFuncName, Q(SC_persian));
      SO_StringCopy(pInternalName, getText(PERSIAN_NAME));
      peType[0] = PTR_DOUBLE;
      peType[1] = PTR_DOUBLE;
      peType[2] = PTR_STRING;
      peType[3] = PTR_DOUBLE;
      *nParamCount = 4;
      break;

    case FUNC_ROMAN:
      SO_StringCopy(pFuncName, Q(SC_latin));
      SO_StringCopy(pInternalName, getText(ROMAN_NAME));
      peType[0] = PTR_DOUBLE;
      peType[1] = PTR_DOUBLE;
      peType[2] = PTR_STRING;
      peType[3] = PTR_STRING;
      peType[4] = PTR_STRING;
      *nParamCount = 5;
      break;

    case FUNC_SELEUCID:
      SO_StringCopy(pFuncName, Q(SC_seleucid));
      SO_StringCopy(pInternalName, getText(SELEUCID_NAME));
      peType[0] = PTR_DOUBLE;
      peType[1] = PTR_DOUBLE;
      peType[2] = PTR_STRING;
      peType[3] = PTR_DOUBLE;
      *nParamCount = 4;
      break;

    case FUNC_TAIWANESE:
      SO_StringCopy(pFuncName, Q(SC_taiwanese));
      SO_StringCopy(pInternalName, getText(TAIWANESE_NAME));
      peType[0] = PTR_DOUBLE;
      peType[1] = PTR_DOUBLE;
      peType[2] = PTR_STRING;
      peType[3] = PTR_DOUBLE;
      *nParamCount = 4;
      break;

    case FUNC_WHEN:
      SO_StringCopy(pFuncName, Q(SC_when));
      SO_StringCopy(pInternalName, getText(WHEN_NAME));
      peType[0] = PTR_DOUBLE;
      peType[1] = PTR_STRING;
      peType[2] = PTR_STRING;
      peType[3] = PTR_STRING;
      peType[4] = PTR_DOUBLE;
      peType[5] = PTR_STRING;
      peType[6] = PTR_DOUBLE;
      *nParamCount = 7;
      break;

    case FUNC_WHEN2:
      SO_StringCopy(pFuncName, Q(SC_when2));
      SO_StringCopy(pInternalName, getText(WHEN2_NAME));
      peType[0] = PTR_DOUBLE;
      peType[1] = PTR_STRING;
      peType[2] = PTR_STRING;
      peType[3] = PTR_STRING;
      peType[4] = PTR_STRING;
      peType[5] = PTR_DOUBLE;
      *nParamCount = 6;
      break;

    case FUNC_SYRIAN:
      SO_StringCopy(pFuncName, Q(SC_syrian));
      SO_StringCopy(pInternalName, getText(SYRIAN_NAME));
      peType[0] = PTR_DOUBLE;
      peType[1] = PTR_DOUBLE;
      peType[2] = PTR_STRING;
      peType[3] = PTR_DOUBLE;
      *nParamCount = 4;
      break;

    case FUNC_YAZDEGERD:
      SO_StringCopy(pFuncName, Q(SC_yazdegerd));
      SO_StringCopy(pInternalName, getText(YAZDEGERD_NAME));
      peType[0] = PTR_DOUBLE;
      peType[1] = PTR_DOUBLE;
      peType[2] = PTR_STRING;
      peType[3] = PTR_DOUBLE;
      *nParamCount = 4;
      break;

    default:
      *nParamCount = *pFuncName = *pInternalName = 0;
  }
}


/*
 * Jede exportierte Funktion erhaelt eine kurze Erlaeuterung was sie tut,
 * welche Argumente sie erwartet usw...
 * Diese Kurzhilfe wird vom Funktionsautopiloten von StarCalc erwartet.
 *
 * Folgende Parameter muessen versorgt werden:
 *
 * USHORT *nNo:	   Nummer der Funktion zwischen 0 und nCount - 1  
 * USHORT *nParam: Nummer der Funktionsparameters
 * char   *pName:  Name des Funktionsparameters
 * char   *pDesc:  Kurze Beschreibung des Parameters
 */
SO_QUIRKSCALL(GetParameterDescription, USHORT *nNo, USHORT *nParam,
				      char *pName, char *pDesc)
{
  *pName = *pDesc = 0;

  switch (*nNo) {

    case FUNC_ARMENIAN:
      switch (*nParam) {
        case 0:
	  SO_StringCopy(pDesc, getText(ARMENIAN_DESC));
	  break;

        case 1:
	  SO_StringCopy(pName, getText(YEAR_NAME));
	  SO_StringCopy(pDesc, getText(YEAR_DESC));
	  break;

        case 2:
	  SO_StringCopy(pName, getText(MONTHNAME_NAME));
	  SO_StringCopy(pDesc, getText(MONTHNAME_DESC));
	  break;

        case 3:
	  SO_StringCopy(pName, getText(DAY_NAME));
	  SO_StringCopy(pDesc, getText(DAY_DESC));
	  break;
      }
      break;

    case FUNC_ASTRONOMY:
      switch (*nParam) {
        case 0:
	  SO_StringCopy(pDesc, getText(ASTRONOMY_DESC));
	  break;

        case 1:
	  SO_StringCopy(pName, getText(DATE_NAME));
	  SO_StringCopy(pDesc, getText(DATE_DESC));
	  break;
      }
      break;

    case FUNC_BUDDHIST:
      switch (*nParam) {
        case 0:
	  SO_StringCopy(pDesc, getText(BUDDHIST_DESC));
	  break;

        case 1:
	  SO_StringCopy(pName, getText(YEAR_NAME));
	  SO_StringCopy(pDesc, getText(YEAR_DESC));
	  break;

        case 2:
	  SO_StringCopy(pName, getText(MONTHNAME_NAME));
	  SO_StringCopy(pDesc, getText(MONTHNAME_DESC));
	  break;

        case 3:
	  SO_StringCopy(pName, getText(DAY_NAME));
	  SO_StringCopy(pDesc, getText(DAY_DESC));
	  break;
      }
      break;

    case FUNC_BYZANTINE:
      switch (*nParam) {
        case 0:
	  SO_StringCopy(pDesc, getText(BYZANTINE_DESC));
	  break;

        case 1:
	  SO_StringCopy(pName, getText(YEAR_NAME));
	  SO_StringCopy(pDesc, getText(YEAR_DESC));
	  break;

        case 2:
	  SO_StringCopy(pName, getText(MONTHNAME_NAME));
	  SO_StringCopy(pDesc, getText(MONTHNAME_DESC));
	  break;

        case 3:
	  SO_StringCopy(pName, getText(DAY_NAME));
	  SO_StringCopy(pDesc, getText(DAY_DESC));
	  break;
      }
      break;

    case FUNC_CALENDAR:
      switch (*nParam) {
        case 0:
	  SO_StringCopy(pDesc, getText(CALENDAR_DESC));
	  break;

        case 1:
	  SO_StringCopy(pName, getText(CALTYPE_NAME));
	  SO_StringCopy(pDesc, getText(CALTYPE_DESC));
	  break;

        case 2:
	  SO_StringCopy(pName, getText(DATE_NAME));
	  SO_StringCopy(pDesc, getText(DATE_DESC));
	  break;

        case 3:
	  SO_StringCopy(pName, getText(CALMODE_NAME));
	  SO_StringCopy(pDesc, getText(CALMODE_DESC));
	  break;
      }
      break;

    case FUNC_CHINESE:
      switch (*nParam) {
        case 0:
	  SO_StringCopy(pDesc, getText(CHINESE_DESC));
	  break;

        case 1:
	  SO_StringCopy(pName, getText(CYCLE_NAME));
	  SO_StringCopy(pDesc, getText(CYCLE_DESC));
	  break;

        case 2:
	  SO_StringCopy(pName, getText(YEAR_NAME));
	  SO_StringCopy(pDesc, getText(YEAR_DESC));
	  break;

        case 3:
	  SO_StringCopy(pName, getText(MONTHNAME_NAME));
	  SO_StringCopy(pDesc, getText(MONTHNAME_DESC));
	  break;

        case 4:
	  SO_StringCopy(pName, getText(LEAPMONTH_NAME));
	  SO_StringCopy(pDesc, getText(LEAPMONTH_DESC));
	  break;

        case 5:
	  SO_StringCopy(pName, getText(DAY_NAME));
	  SO_StringCopy(pDesc, getText(DAY_DESC));
	  break;
      }
      break;

    case FUNC_CHRONOLOGY:
      switch (*nParam) {
        case 0:
	  SO_StringCopy(pDesc, getText(CHRONOLOGY_DESC));
	  break;

        case 1:
	  SO_StringCopy(pName, getText(CALCYCLE_NAME));
	  SO_StringCopy(pDesc, getText(CALCYCLE_DESC));
	  break;

        case 2:
	  SO_StringCopy(pName, getText(YEAR_NAME));
	  SO_StringCopy(pDesc, getText(YEAR_DESC));
	  break;
      }
      break;

    case FUNC_COPTIC:
      switch (*nParam) {
        case 0:
	  SO_StringCopy(pDesc, getText(COPTIC_DESC));
	  break;

        case 1:
	  SO_StringCopy(pName, getText(YEAR_NAME));
	  SO_StringCopy(pDesc, getText(YEAR_DESC));
	  break;

        case 2:
	  SO_StringCopy(pName, getText(MONTHNAME_NAME));
	  SO_StringCopy(pDesc, getText(MONTHNAME_DESC));
	  break;

        case 3:
	  SO_StringCopy(pName, getText(DAY_NAME));
	  SO_StringCopy(pDesc, getText(DAY_DESC));
	  break;
      }
      break;

    case FUNC_EGYPTIAN:
      switch (*nParam) {
        case 0:
	  SO_StringCopy(pDesc, getText(EGYPTIAN_DESC));
	  break;

        case 1:
	  SO_StringCopy(pName, getText(YEAR_NAME));
	  SO_StringCopy(pDesc, getText(YEAR_DESC));
	  break;

        case 2:
	  SO_StringCopy(pName, getText(MONTHNAME_NAME));
	  SO_StringCopy(pDesc, getText(MONTHNAME_DESC));
	  break;

        case 3:
	  SO_StringCopy(pName, getText(DAY_NAME));
	  SO_StringCopy(pDesc, getText(DAY_DESC));
	  break;
      }
      break;

    case FUNC_ETHIOPIC:
      switch (*nParam) {
        case 0:
	  SO_StringCopy(pDesc, getText(ETHIOPIC_DESC));
	  break;

        case 1:
	  SO_StringCopy(pName, getText(YEAR_NAME));
	  SO_StringCopy(pDesc, getText(YEAR_DESC));
	  break;

        case 2:
	  SO_StringCopy(pName, getText(MONTHNAME_NAME));
	  SO_StringCopy(pDesc, getText(MONTHNAME_DESC));
	  break;

        case 3:
	  SO_StringCopy(pName, getText(DAY_NAME));
	  SO_StringCopy(pDesc, getText(DAY_DESC));
	  break;
      }
      break;

    case FUNC_EVENT:
      switch (*nParam) {
        case 0:
	  SO_StringCopy(pDesc, getText(EVENT_DESC));
	  break;

        case 1:
	  SO_StringCopy(pName, getText(ORDINAL_NAME));
	  SO_StringCopy(pDesc, getText(ORDINAL_DESC));
	  break;

        case 2:
	  SO_StringCopy(pName, getText(DAYNAME_NAME));
	  SO_StringCopy(pDesc, getText(DAYNAME_DESC));
	  break;

        case 3:
	  SO_StringCopy(pName, getText(MONTHNAME_NAME));
	  SO_StringCopy(pDesc, getText(MONTHNAME_DESC));
	  break;

        case 4:
	  SO_StringCopy(pName, getText(YEAR_NAME));
	  SO_StringCopy(pDesc, getText(YEAR_DESC));
	  break;
      }
      break;

    case FUNC_FRENCH:
      switch (*nParam) {
        case 0:
	  SO_StringCopy(pDesc, getText(FRENCH_DESC));
	  break;

        case 1:
	  SO_StringCopy(pName, getText(YEAR_NAME));
	  SO_StringCopy(pDesc, getText(YEAR_DESC));
	  break;

        case 2:
	  SO_StringCopy(pName, getText(MONTHNAME_NAME));
	  SO_StringCopy(pDesc, getText(MONTHNAME_DESC));
	  break;

        case 3:
	  SO_StringCopy(pName, getText(DAY_NAME));
	  SO_StringCopy(pDesc, getText(DAY_DESC));
	  break;
      }
      break;

    case FUNC_GENGOU:
      switch (*nParam) {
        case 0:
	  SO_StringCopy(pDesc, getText(GENGOU_DESC));
	  break;

        case 1:
	  SO_StringCopy(pName, getText(GENGOU_ERA_NAME));
	  SO_StringCopy(pDesc, getText(GENGOU_ERA_DESC));
	  break;

        case 2:
	  SO_StringCopy(pName, getText(YEAR_NAME));
	  SO_StringCopy(pDesc, getText(YEAR_DESC));
	  break;

        case 3:
	  SO_StringCopy(pName, getText(MONTHNAME_NAME));
	  SO_StringCopy(pDesc, getText(MONTHNAME_DESC));
	  break;

        case 4:
	  SO_StringCopy(pName, getText(DAY_NAME));
	  SO_StringCopy(pDesc, getText(DAY_DESC));
	  break;
      }
      break;

    case FUNC_GNOMON:
      switch (*nParam) {
        case 0:
	  SO_StringCopy(pDesc, getText(GNOMON_DESC));
	  break;

        case 1:
	  SO_StringCopy(pName, getText(DATE_NAME));
	  SO_StringCopy(pDesc, getText(DATE_DESC));
	  break;
      }
      break;

    case FUNC_GREGORIAN:
      switch (*nParam) {
        case 0:
	  SO_StringCopy(pDesc, getText(GREGORIAN_DESC));
	  break;

        case 1:
	  SO_StringCopy(pName, getText(YEAR_NAME));
	  SO_StringCopy(pDesc, getText(YEAR_DESC));
	  break;

        case 2:
	  SO_StringCopy(pName, getText(MONTHNAME_NAME));
	  SO_StringCopy(pDesc, getText(MONTHNAME_DESC));
	  break;

        case 3:
	  SO_StringCopy(pName, getText(DAY_NAME));
	  SO_StringCopy(pDesc, getText(DAY_DESC));
	  break;
      }
      break;

    case FUNC_HEBREW:
      switch (*nParam) {
        case 0:
	  SO_StringCopy(pDesc, getText(HEBREW_DESC));
	  break;

        case 1:
	  SO_StringCopy(pName, getText(YEAR_NAME));
	  SO_StringCopy(pDesc, getText(YEAR_DESC));
	  break;

        case 2:
	  SO_StringCopy(pName, getText(MONTHNAME_NAME));
	  SO_StringCopy(pDesc, getText(MONTHNAME_DESC));
	  break;

        case 3:
	  SO_StringCopy(pName, getText(DAY_NAME));
	  SO_StringCopy(pDesc, getText(DAY_DESC));
	  break;
      }
      break;

    case FUNC_HINDU:
      switch (*nParam) {
        case 0:
	  SO_StringCopy(pDesc, getText(HINDU_DESC));
	  break;

        case 1:
	  SO_StringCopy(pName, getText(YEAR_NAME));
	  SO_StringCopy(pDesc, getText(YEAR_DESC));
	  break;

        case 2:
	  SO_StringCopy(pName, getText(MONTHNAME_NAME));
	  SO_StringCopy(pDesc, getText(MONTHNAME_DESC));
	  break;

        case 3:
	  SO_StringCopy(pName, getText(LEAPMONTH_NAME));
	  SO_StringCopy(pDesc, getText(LEAPMONTH_DESC));
	  break;

        case 4:
	  SO_StringCopy(pName, getText(DAY_NAME));
	  SO_StringCopy(pDesc, getText(DAY_DESC));
	  break;
      }
      break;

    case FUNC_HINDU2:
      switch (*nParam) {
        case 0:
	  SO_StringCopy(pDesc, getText(HINDU2_DESC));
	  break;

        case 1:
	  SO_StringCopy(pName, getText(YEAR_NAME));
	  SO_StringCopy(pDesc, getText(YEAR_DESC));
	  break;

        case 2:
	  SO_StringCopy(pName, getText(MONTHNAME_NAME));
	  SO_StringCopy(pDesc, getText(MONTHNAME_DESC));
	  break;

        case 3:
	  SO_StringCopy(pName, getText(DAY_NAME));
	  SO_StringCopy(pDesc, getText(DAY_DESC));
	  break;
      }
      break;

    case FUNC_HOLIDAY:
      switch (*nParam) {
        case 0:
	  SO_StringCopy(pDesc, getText(HOLIDAY_DESC));
	  break;

        case 1:
	  SO_StringCopy(pName, getText(CALTYPE_NAME));
	  SO_StringCopy(pDesc, getText(CALTYPE_DESC));
	  break;

        case 2:
	  SO_StringCopy(pName, getText(HOLIDAYS_NAME));
	  SO_StringCopy(pDesc, getText(HOLIDAYS_DESC));
	  break;

        case 3:
	  SO_StringCopy(pName, getText(EPOCH_NAME));
	  SO_StringCopy(pDesc, getText(EPOCH_DESC));
	  break;
      }
      break;

    case FUNC_HOLIDAY2:
      switch (*nParam) {
        case 0:
	  SO_StringCopy(pDesc, getText(HOLIDAY2_DESC));
	  break;

        case 1:
	  SO_StringCopy(pName, getText(CALTYPE_NAME));
	  SO_StringCopy(pDesc, getText(CALTYPE_DESC));
	  break;

        case 2:
	  SO_StringCopy(pName, getText(HOLIDAYS_NAME));
	  SO_StringCopy(pDesc, getText(HOLIDAYS_DESC));
	  break;

        case 3:
	  SO_StringCopy(pName, getText(START_NAME));
	  SO_StringCopy(pDesc, getText(START_DESC));
	  break;
      }
      break;

    case FUNC_INDIAN:
      switch (*nParam) {
        case 0:
	  SO_StringCopy(pDesc, getText(INDIAN_DESC));
	  break;

        case 1:
	  SO_StringCopy(pName, getText(YEAR_NAME));
	  SO_StringCopy(pDesc, getText(YEAR_DESC));
	  break;

        case 2:
	  SO_StringCopy(pName, getText(MONTHNAME_NAME));
	  SO_StringCopy(pDesc, getText(MONTHNAME_DESC));
	  break;

        case 3:
	  SO_StringCopy(pName, getText(DAY_NAME));
	  SO_StringCopy(pDesc, getText(DAY_DESC));
	  break;
      }
      break;

    case FUNC_ISLAMIC:
      switch (*nParam) {
        case 0:
	  SO_StringCopy(pDesc, getText(ISLAMIC_DESC));
	  break;

        case 1:
	  SO_StringCopy(pName, getText(YEAR_NAME));
	  SO_StringCopy(pDesc, getText(YEAR_DESC));
	  break;

        case 2:
	  SO_StringCopy(pName, getText(MONTHNAME_NAME));
	  SO_StringCopy(pDesc, getText(MONTHNAME_DESC));
	  break;

        case 3:
	  SO_StringCopy(pName, getText(DAY_NAME));
	  SO_StringCopy(pDesc, getText(DAY_DESC));
	  break;

      }
      break;

    case FUNC_ISO8601:
      switch (*nParam) {
        case 0:
	  SO_StringCopy(pDesc, getText(ISO8601_DESC));
	  break;

        case 1:
	  SO_StringCopy(pName, getText(YEAR_NAME));
	  SO_StringCopy(pDesc, getText(YEAR_DESC));
	  break;

        case 2:
	  SO_StringCopy(pName, getText(WEEK_NAME));
	  SO_StringCopy(pDesc, getText(WEEK_DESC));
	  break;

        case 3:
	  SO_StringCopy(pName, getText(DAY_NAME));
	  SO_StringCopy(pDesc, getText(DAY_DESC));
	  break;

      }
      break;

    case FUNC_JULIAN:
      switch (*nParam) {
        case 0:
	  SO_StringCopy(pDesc, getText(JULIAN_DESC));
	  break;

        case 1:
	  SO_StringCopy(pName, getText(YEAR_NAME));
	  SO_StringCopy(pDesc, getText(YEAR_DESC));
	  break;

        case 2:
	  SO_StringCopy(pName, getText(MONTHNAME_NAME));
	  SO_StringCopy(pDesc, getText(MONTHNAME_DESC));
	  break;

        case 3:
	  SO_StringCopy(pName, getText(DAY_NAME));
	  SO_StringCopy(pDesc, getText(DAY_DESC));
	  break;
      }
      break;

    case FUNC_LEAPYEAR:
      switch (*nParam) {
        case 0:
	  SO_StringCopy(pDesc, getText(LEAPYEAR_DESC));
	  break;

        case 1:
	  SO_StringCopy(pName, getText(CALTYPE_NAME));
	  SO_StringCopy(pDesc, getText(CALTYPE_DESC));
	  break;

        case 2:
	  SO_StringCopy(pName, getText(EPOCH_NAME));
	  SO_StringCopy(pDesc, getText(EPOCH_DESC));
	  break;
      }
      break;

    case FUNC_LEAPYEAR2:
      switch (*nParam) {
        case 0:
	  SO_StringCopy(pDesc, getText(LEAPYEAR2_DESC));
	  break;

        case 1:
	  SO_StringCopy(pName, getText(CALTYPE_NAME));
	  SO_StringCopy(pDesc, getText(CALTYPE_DESC));
	  break;

        case 2:
	  SO_StringCopy(pName, getText(DATE_NAME));
	  SO_StringCopy(pDesc, getText(DATE_DESC));
	  break;
      }
      break;

    case FUNC_MAYAN:
      switch (*nParam) {
        case 0:
	  SO_StringCopy(pDesc, getText(MAYAN_DESC));
	  break;

        case 1:
	  SO_StringCopy(pName, getText(BAKTUN_NAME));
	  SO_StringCopy(pDesc, getText(BAKTUN_DESC));
	  break;

        case 2:
	  SO_StringCopy(pName, getText(KATUN_NAME));
	  SO_StringCopy(pDesc, getText(KATUN_DESC));
	  break;

        case 3:
	  SO_StringCopy(pName, getText(TUN_NAME));
	  SO_StringCopy(pDesc, getText(TUN_DESC));
	  break;

        case 4:
	  SO_StringCopy(pName, getText(UINAL_NAME));
	  SO_StringCopy(pDesc, getText(UINAL_DESC));
	  break;

        case 5:
	  SO_StringCopy(pName, getText(KIN_NAME));
	  SO_StringCopy(pDesc, getText(KIN_DESC));
	  break;
      }
      break;

    case FUNC_PERSIAN:
      switch (*nParam) {
        case 0:
	  SO_StringCopy(pDesc, getText(PERSIAN_DESC));
	  break;

        case 1:
	  SO_StringCopy(pName, getText(YEAR_NAME));
	  SO_StringCopy(pDesc, getText(YEAR_DESC));
	  break;

        case 2:
	  SO_StringCopy(pName, getText(MONTHNAME_NAME));
	  SO_StringCopy(pDesc, getText(MONTHNAME_DESC));
	  break;

        case 3:
	  SO_StringCopy(pName, getText(DAY_NAME));
	  SO_StringCopy(pDesc, getText(DAY_DESC));
	  break;
      }
      break;

    case FUNC_ROMAN:
      switch (*nParam) {
        case 0:
	  SO_StringCopy(pDesc, getText(ROMAN_DESC));
	  break;

        case 1:
	  SO_StringCopy(pName, getText(YEAR_NAME));
	  SO_StringCopy(pDesc, getText(YEAR_DESC));
	  break;

        case 2:
	  SO_StringCopy(pName, getText(MONTHNAME_NAME));
	  SO_StringCopy(pDesc, getText(MONTHNAME_DESC));
	  break;

        case 3:
	  SO_StringCopy(pName, getText(FIXPOINT_NAME));
	  SO_StringCopy(pDesc, getText(FIXPOINT_DESC));
	  break;

        case 4:
	  SO_StringCopy(pName, getText(DAY_NAME));
	  SO_StringCopy(pDesc, getText(DAY_DESC));
	  break;
      }
      break;

    case FUNC_SELEUCID:
      switch (*nParam) {
        case 0:
	  SO_StringCopy(pDesc, getText(SELEUCID_DESC));
	  break;

        case 1:
	  SO_StringCopy(pName, getText(YEAR_NAME));
	  SO_StringCopy(pDesc, getText(YEAR_DESC));
	  break;

        case 2:
	  SO_StringCopy(pName, getText(MONTHNAME_NAME));
	  SO_StringCopy(pDesc, getText(MONTHNAME_DESC));
	  break;

        case 3:
	  SO_StringCopy(pName, getText(DAY_NAME));
	  SO_StringCopy(pDesc, getText(DAY_DESC));
	  break;
      }
      break;

    case FUNC_TAIWANESE:
      switch (*nParam) {
        case 0:
	  SO_StringCopy(pDesc, getText(TAIWANESE_DESC));
	  break;

        case 1:
	  SO_StringCopy(pName, getText(YEAR_NAME));
	  SO_StringCopy(pDesc, getText(YEAR_DESC));
	  break;

        case 2:
	  SO_StringCopy(pName, getText(MONTHNAME_NAME));
	  SO_StringCopy(pDesc, getText(MONTHNAME_DESC));
	  break;

        case 3:
	  SO_StringCopy(pName, getText(DAY_NAME));
	  SO_StringCopy(pDesc, getText(DAY_DESC));
	  break;
      }
      break;

    case FUNC_WHEN:
      switch (*nParam) {
        case 0:
	  SO_StringCopy(pDesc, getText(WHEN_DESC));
	  break;

        case 1:
	  SO_StringCopy(pName, getText(CALTYPE_NAME));
	  SO_StringCopy(pDesc, getText(CALTYPE_DESC));
	  break;

        case 2:
	  SO_StringCopy(pName, getText(HOLIDAYS_NAME));
	  SO_StringCopy(pDesc, getText(HOLIDAYS_DESC));
	  break;

        case 3:
	  SO_StringCopy(pName, getText(DAYNAME_NAME));
	  SO_StringCopy(pDesc, getText(DAYNAME_DESC));
	  break;

        case 4:
	  SO_StringCopy(pName, getText(DAY_NAME));
	  SO_StringCopy(pDesc, getText(DAY_DESC));
	  break;

        case 5:
	  SO_StringCopy(pName, getText(MONTH_NAME));
	  SO_StringCopy(pDesc, getText(MONTH_DESC));
	  break;

        case 6:
	  SO_StringCopy(pName, getText(START_NAME));
	  SO_StringCopy(pDesc, getText(START_DESC));
	  break;
      }
      break;

    case FUNC_WHEN2:
      switch (*nParam) {
        case 0:
	  SO_StringCopy(pDesc, getText(WHEN2_DESC));
	  break;

        case 1:
	  SO_StringCopy(pName, getText(CALTYPE1_NAME));
	  SO_StringCopy(pDesc, getText(CALTYPE_DESC));
	  break;

        case 2:
	  SO_StringCopy(pName, getText(HOLIDAYS1_NAME));
	  SO_StringCopy(pDesc, getText(HOLIDAYS_DESC));
	  break;

        case 3:
	  SO_StringCopy(pName, getText(CALTYPE2_NAME));
	  SO_StringCopy(pDesc, getText(CALTYPE_DESC));
	  break;

        case 4:
	  SO_StringCopy(pName, getText(HOLIDAYS2_NAME));
	  SO_StringCopy(pDesc, getText(HOLIDAYS_DESC));
	  break;

        case 5:
	  SO_StringCopy(pName, getText(START_NAME));
	  SO_StringCopy(pDesc, getText(START_DESC));
	  break;
      }
      break;

    case FUNC_SYRIAN:
      switch (*nParam) {
        case 0:
	  SO_StringCopy(pDesc, getText(SYRIAN_DESC));
	  break;

        case 1:
	  SO_StringCopy(pName, getText(YEAR_NAME));
	  SO_StringCopy(pDesc, getText(YEAR_DESC));
	  break;

        case 2:
	  SO_StringCopy(pName, getText(MONTHNAME_NAME));
	  SO_StringCopy(pDesc, getText(MONTHNAME_DESC));
	  break;

        case 3:
	  SO_StringCopy(pName, getText(DAY_NAME));
	  SO_StringCopy(pDesc, getText(DAY_DESC));
	  break;
      }
      break;

    case FUNC_YAZDEGERD:
      switch (*nParam) {
        case 0:
	  SO_StringCopy(pDesc, getText(YAZDEGERD_DESC));
	  break;

        case 1:
	  SO_StringCopy(pName, getText(YEAR_NAME));
	  SO_StringCopy(pDesc, getText(YEAR_DESC));
	  break;

        case 2:
	  SO_StringCopy(pName, getText(MONTHNAME_NAME));
	  SO_StringCopy(pDesc, getText(MONTHNAME_DESC));
	  break;

        case 3:
	  SO_StringCopy(pName, getText(DAY_NAME));
	  SO_StringCopy(pDesc, getText(DAY_DESC));
	  break;
      }
      break;

  }
}

