/*
 * CALENDAR is distributed under the GNU GENERAL PUBLIC LICENSE (GPL)
 * Version 2 (June 1991). See the "COPYING" file distributed with this
 * software for more info.
 *
 * Copyright (C) 2000 - 2003 by Michael Schroeder <mdhs@imsch.de>
 */

#include "fmath.h"
#include "astro.h"
#include "stdlib.h"
#include "string.h"


#include "starcalc.h"
#include "calendar.h"


#define SKIP	0x01


/*
 * Sucht einen Text in einer Auswahl von Strings und liefert den zugehoerigen
 * Zeiger (falls vorhanden), sonst den Index i (beginnend mit 1) zurueck.
 *
 * - Es werden die Strings der eingestellten Sprache (_nLanguage) durchsucht.
 *
 * - Es werden nur die Anzahl der Zeichen in 'txt' mit den Strings verglichen.
 *
 * - Falls 'txt' aus Ziffern besteht, werden diese als Index gewertet.
 *
 * - Wenn die Zeiger-Liste leer ist, dann wird der Index i zuruckgeliefert. 
 *
 * - Ein Zeiger von (Null) bzw. Index von (0) bedeuten: 'nicht vorhanden'.
 */
void *scan(const char *txt, const FUNCLIST *list)
{
  int i, l;
  const STRING *ptr;

  i = atoi(txt);

  if (i <= 0 || i > list->count) {	/* keine oder falsche Ziffern ? */

    switch (_nLanguage) {		/* die richtige Sprache einstellen */
      case LANGUAGE_GERMAN:
        ptr = list->names_de;
        break;

      default:  
        ptr = list->names;
    }

    if (*txt == '\0') {				/* leerer Eintrag ? */
      for (i = list->count; i > 0; i--)
        if (*ptr[i-1] == '\0')
	  break;
    }
    else {	/* Wichtig: lexikalisch aufsteigend suchen! */

      for (l = strlen(txt), i = 1; i <= list->count; i++) {
        if (strchk(txt, ptr[i-1], l) == 0)
          break;
      }
      if (i > list->count)	/* nicht gefunden :-( */
        i = 0;
    }
  }

  if (i)			/* nicht 0 ? */
    if(list->funcs)		/* ist Zeiger-Liste vorhanden ? */
      return list->funcs[i-1];	/* Funktion zurueckliefern */

  return (void*)i;
}


/*
 * Umwandlung in Kleinbuchstaben. Alle nicht-alphanumerischen Zeichen
 * bekommen den Code SKIP.
 */
#ifdef LATIN1
const unsigned char __conv[256] =
{
  /* 0x00 */ 0x00, SKIP, SKIP, SKIP, SKIP, SKIP, SKIP, SKIP,
  /* 0x08 */ SKIP, SKIP, SKIP, SKIP, SKIP, SKIP, SKIP, SKIP,
  /* 0x10 */ SKIP, SKIP, SKIP, SKIP, SKIP, SKIP, SKIP, SKIP,
  /* 0x18 */ SKIP, SKIP, SKIP, SKIP, SKIP, SKIP, SKIP, SKIP,
  /* 0x20 */ SKIP, SKIP, SKIP, SKIP, SKIP, SKIP, SKIP, SKIP,
  /* 0x28 */ SKIP, SKIP, SKIP, SKIP, SKIP, SKIP, SKIP, SKIP,
  /* 0x30 */ 0x30, 0x31, 0x32, 0x33, 0x34, 0x35, 0x36, 0x37,
  /* 0x38 */ 0x38, 0x39, SKIP, SKIP, SKIP, SKIP, SKIP, SKIP,
  /* 0x40 */ SKIP, 0x61, 0x62, 0x63, 0x64, 0x65, 0x66, 0x67,
  /* 0x48 */ 0x68, 0x69, 0x6a, 0x6b, 0x6c, 0x6d, 0x6e, 0x6f,
  /* 0x50 */ 0x70, 0x71, 0x72, 0x73, 0x74, 0x75, 0x76, 0x77,
  /* 0x58 */ 0x78, 0x79, 0x7a, SKIP, SKIP, SKIP, SKIP, SKIP,
  /* 0x60 */ SKIP, 0x61, 0x62, 0x63, 0x64, 0x65, 0x66, 0x67,
  /* 0x68 */ 0x68, 0x69, 0x6a, 0x6b, 0x6c, 0x6d, 0x6e, 0x6f,
  /* 0x70 */ 0x70, 0x71, 0x72, 0x73, 0x74, 0x75, 0x76, 0x77,
  /* 0x78 */ 0x78, 0x79, 0x7a, SKIP, SKIP, SKIP, SKIP, SKIP,
  /* 0x80 */ SKIP, SKIP, SKIP, SKIP, SKIP, SKIP, SKIP, SKIP,
  /* 0x88 */ SKIP, SKIP, SKIP, SKIP, SKIP, SKIP, SKIP, SKIP,
  /* 0x90 */ SKIP, SKIP, SKIP, SKIP, SKIP, SKIP, SKIP, SKIP,
  /* 0x98 */ SKIP, SKIP, SKIP, SKIP, SKIP, SKIP, SKIP, SKIP,
  /* 0xa0 */ SKIP, SKIP, SKIP, SKIP, SKIP, SKIP, 0xa8, SKIP,
  /* 0xa8 */ 0xa8, SKIP, SKIP, SKIP, SKIP, SKIP, SKIP, SKIP,
  /* 0xb0 */ SKIP, SKIP, SKIP, SKIP, 0xb4, SKIP, SKIP, SKIP,
  /* 0xb8 */ 0xb8, SKIP, SKIP, SKIP, 0xbd, 0xbd, 0xff, SKIP,
  /* 0xc0 */ 0xe0, 0xe1, 0xe2, 0xe3, 0xe4, 0xe5, 0xe6, 0xe7,
  /* 0xc8 */ 0xe8, 0xe9, 0xea, 0xeb, 0xec, 0xed, 0xee, 0xef,
  /* 0xd0 */ 0xf0, 0xf1, 0xf2, 0xf3, 0xf4, 0xf5, 0xf6, SKIP,
  /* 0xd8 */ 0xf8, 0xf9, 0xfa, 0xfb, 0xfc, 0xfd, 0xfe, 0xdf,
  /* 0xe0 */ 0xe0, 0xe1, 0xe2, 0xe3, 0xe4, 0xe5, 0xe6, 0xe7,
  /* 0xe8 */ 0xe8, 0xe9, 0xea, 0xeb, 0xec, 0xed, 0xee, 0xef,
  /* 0xf0 */ 0xf0, 0xf1, 0xf2, 0xf3, 0xf4, 0xf5, 0xf6, SKIP,
  /* 0xf8 */ 0xf8, 0xf9, 0xfa, 0xfb, 0xfc, 0xfd, 0xfe, 0xff,
};

#else		/* CP-850 (CP-437)*/

const unsigned char __conv[256] =
{
  /* 0x00 */ 0x00, SKIP, SKIP, SKIP, SKIP, SKIP, SKIP, SKIP,
  /* 0x08 */ SKIP, SKIP, SKIP, SKIP, SKIP, SKIP, SKIP, SKIP,
  /* 0x10 */ SKIP, SKIP, SKIP, SKIP, SKIP, SKIP, SKIP, SKIP,
  /* 0x18 */ SKIP, SKIP, SKIP, SKIP, SKIP, SKIP, SKIP, SKIP,
  /* 0x20 */ SKIP, SKIP, SKIP, SKIP, SKIP, SKIP, SKIP, SKIP,
  /* 0x28 */ SKIP, SKIP, SKIP, SKIP, SKIP, SKIP, SKIP, SKIP,
  /* 0x30 */ 0x30, 0x31, 0x32, 0x33, 0x34, 0x35, 0x36, 0x37,
  /* 0x38 */ 0x38, 0x39, SKIP, SKIP, SKIP, SKIP, SKIP, SKIP,
  /* 0x40 */ SKIP, 0x61, 0x62, 0x63, 0x64, 0x65, 0x66, 0x67,
  /* 0x48 */ 0x68, 0x69, 0x6a, 0x6b, 0x6c, 0x6d, 0x6e, 0x6f,
  /* 0x50 */ 0x70, 0x71, 0x72, 0x73, 0x74, 0x75, 0x76, 0x77,
  /* 0x58 */ 0x78, 0x79, 0x7a, SKIP, SKIP, SKIP, SKIP, SKIP,
  /* 0x60 */ SKIP, 0x61, 0x62, 0x63, 0x64, 0x65, 0x66, 0x67,
  /* 0x68 */ 0x68, 0x69, 0x6a, 0x6b, 0x6c, 0x6d, 0x6e, 0x6f,
  /* 0x70 */ 0x70, 0x71, 0x72, 0x73, 0x74, 0x75, 0x76, 0x77,
  /* 0x78 */ 0x78, 0x79, 0x7a, SKIP, SKIP, SKIP, SKIP, SKIP,
  /* 0x80 */ 0x87, 0x81, 0x82, 0x83, 0x84, 0x85, 0x86, 0x87,
  /* 0x88 */ 0x88, 0x89, 0x8a, 0x8b, 0x82, 0x8d, 0x84, 0x86,
  /* 0x90 */ 0x82, 0x91, 0x91, 0x93, 0x94, 0x95, 0x96, 0x97,
  /* 0x98 */ 0x98, 0x94, 0x81, SKIP, SKIP, SKIP, SKIP, SKIP,
  /* 0xa0 */ 0xa0, 0xa1, 0xa2, 0xa3, 0xa4, 0xa4, SKIP, SKIP,
  /* 0xa8 */ SKIP, SKIP, SKIP, SKIP, SKIP, SKIP, SKIP, SKIP,
  /* 0xb0 */ SKIP, SKIP, SKIP, SKIP, SKIP, SKIP, SKIP, SKIP,
  /* 0xb8 */ SKIP, SKIP, SKIP, SKIP, SKIP, SKIP, SKIP, SKIP,
  /* 0xc0 */ SKIP, SKIP, SKIP, SKIP, SKIP, SKIP, SKIP, SKIP,
  /* 0xc8 */ SKIP, SKIP, SKIP, SKIP, SKIP, SKIP, SKIP, SKIP,
  /* 0xd0 */ SKIP, SKIP, SKIP, SKIP, SKIP, SKIP, SKIP, SKIP,
  /* 0xd8 */ SKIP, SKIP, SKIP, SKIP, SKIP, SKIP, SKIP, SKIP,
  /* 0xe0 */ 0xa2, 0xe1, 0x93, 0x95, 0xe4, 0xe4, SKIP, 0xe7,
  /* 0xe8 */ 0xe7, 0xa3, 0x96, 0x97, 0xec, 0xec, SKIP, SKIP,
  /* 0xf0 */ SKIP, SKIP, SKIP, SKIP, SKIP, SKIP, SKIP, SKIP,
  /* 0xf8 */ SKIP, SKIP, SKIP, SKIP, SKIP, SKIP, SKIP, SKIP,
};
#endif

/*
 * Strings vergleichen.
 *
 * Die Funktion arbeitet wie strncasecmp() mit dem Unterschied,
 * dass auch lokale Zeichen (Umlaute ...) korrekt verarbeitet werden
 * und alle anderen (nicht alpha-numerischen) Zeichen beim Vergleich
 * bersprungen werden.
 *
 * Beispiel (die folgenden Zeichenketten sind gleichwertig):
 *
 *	"Marsopposition"
 *	"MARSOPPOSITION"
 *	"Mars-Opposition"
 *	"Mars' Opposition"
 */
int strchk(const char *s1, const char *s2, int n)
{
  const unsigned char *p1 = (const unsigned char *)s1;
  const unsigned char *p2 = (const unsigned char *)s2;
  unsigned char c1, c2;

  if (p1 == p2 || n == 0)
    return 0;

  do {
      while ((c1 = __conv[*p1++]) == SKIP);
      while ((c2 = __conv[*p2++]) == SKIP) n--;
//      c1 = __conv[*p1++];
//      c2 = __conv[*p2++];

      if (c1 == '\0' || c1 != c2)
	return c1 - c2;

  } while (--n > 0);

  return c1 - c2;
}


#ifdef DEBUG
int main (void)
{
  int i;
  
    for (i=0; i<256; i++)
      printf("i:%2d (%c) -> (%c)\n", i, i, __conv[i]);

  return 0;
}
#endif
