/* $Id$ */
/*-----------------------------------------------------*/
/*     Name:               version.c                   */
/*-----------------------------------------------------*/
/*  Aufgabe: Versionsinformationen                     */
/*-----------------------------------------------------*/
/*   Copyright (C) 1995 - 2003 by Michael Schroeder    */
/*-----------------------------------------------------*/


/*
 * CALENDAR is distributed under the GNU GENERAL PUBLIC LICENSE (GPL)
 * Version 2 (June 1991). See the "COPYING" file distributed with this
 * software for more info.
 */

/*
 * Die Funktion VERSIONINFO() kann in einer DLL auf einfache Weise mittels REXX
 * (oder einer entsprechenden Sprache) abgefragt werden.
 *
 * BASIC: (z.B. StarOffice)
 *	Declare Function versioninfo Lib "....dll" (ByVal info As Long ) As String
 *	...
 *	Print Versioninfo(3)
 *	...
 *	FreeLibrary("....dll")
 *
 * OS/2 (REXX):
 *	Call RxFuncAdd Versioninfo, ....dll, Versioninfo
 *	...
 *	Say Versioninfo(3)
 *	...
 *	Call RxFuncDrop Versioninfo
 *
 * Allgemein: (z.B. durch Laden der DLL mittels eines C-Programms)
 *
 *
 * - Die Schreibweise von 'versioninfo' ist hierbei zu beachten!
 *
 * - REXX verwendet den DLL-Einsprungpunkt in Grossbuchstaben.
 *   Dagegen ist die Schreibweise in den REXX-Scripten belanglos.
 *
 * - BASIC erwartet zumindest bei der Deklaration die exakte Schreibweise.
 */

#include "version.h"

#define ENTRIES(x)	(sizeof(x) / sizeof(*x))


static const char *const verinfo[] =
{
  (char*)BUILD,
  NAME,
  LIBPRO PROGRAM LIBEXT,
  FILEVER,
  COPYRIGHT,
  LICENSE,
  COMPANY,
  DESCRIPTION,
  PRODUCT,
  MACHINE,
  COMPILER,
  OPTFLAGS,
};


const char* versioninfo(int m)
{
  return verinfo[m];
}


long VERSIONINFO(const char *name, long argc, const XSTRING argv[],
             const char *queue, pXSTRING ret)
{
  int i, n = VER_FILEVERSION;

  if (argc) {
    if (argc > 1)
      return 1;

    n = *argv[0].ptr - '0';

    if (argv[0].ptr[1] || n < 1 || n > ENTRIES(verinfo) - 1)
      return 1;
  }

  for (i = 0; verinfo[n][i] != 0; i++)
    ret->ptr[i] = verinfo[n][i];

  ret->len = i;

  return 0;
}
